@@ -1,3 +1,4 @@
+from __future__ import annotations
 import numpy as np
 import warnings
 import time
@@ -16,17 +17,14 @@ from .enum import Color
 if TYPE_CHECKING:
     from .player import Player
 
-cmap = ListedColormap(["white", (0.59, 0.44, 0.2), "black"])
+cmap = ListedColormap(["red", (0.59, 0.44, 0.2), "black", "white", "green", "blue", "yellow", "purple"])
+color_list = [c.value for c in Color]
+max_color = max(color_list)
+min_color = min(color_list)
 
 
 class Board:
-    """ Represents the goban of a game
-
-    :Parameters:
-        size: Union[:class:`int`, tuple[:class:`int`, :class:`int`]]
-            The size of the board, either an int for a square board, or a tuple (height, width)
-        show: :class:`int`
-            Indicates if the board should be displayed after each move. Default to `False`
+    """Represents the goban of a game
 
     :Notes:
         Board vertices can be accessed through indices:
@@ -36,6 +34,10 @@ class Board:
         <Color.Empty: 0>
     """
     def __init__(self, *, size: Union[int, tuple[int, int]] = 19, show: bool = False):
+        """
+        :param size: The size of the board, either an int for a square board, or a tuple (height, width)
+
+        :param show: Indicates if the board should be displayed after each move"""
         if isinstance(size, int):
             height, width = size, size
         elif isinstance(size, (tuple, list, np.ndarray)):
@@ -48,10 +50,26 @@ class Board:
         self.show: bool = show
         self._grid: np.ndarray = np.full((height, width), Color.Empty)
         self._last_grid: np.ndarray = np.copy(self._grid)
-        self._next_player: Color = Color.Black
-        self._prisoners: dict[Color, int] = {Color.Black: 0, Color.White: 0}
+        self._current_player: Optional[Player] = None
         self._territories: list[Territory] = [Territory(x=0, y=0, board=self)]
-        self._players: dict[Color, Optional[Player]] = {Color.Black: None, Color.White: None}
+        self._players: dict[Color, Player] = {}
+        self._prisoners: dict[Color, int] = {}
+
+    @classmethod
+    def circular(cls, size: Union[int, tuple[int, int]] = 19, show: bool = False) -> Board:
+        """A quick way to generate a circular board using walls
+
+        :param size: An int denoting the diameter of the circle, or a 2-tuple denoting the two diameter of an oval
+        :param show: Indicates if the board should be displayed after each move"""
+        board = cls(size=size, show=show)
+        middle_x = board._grid.shape[0]/2
+        middle_y = board._grid.shape[1]/2
+        for x in range(board._grid.shape[0]):
+            for y in range(board._grid.shape[1]):
+                if (((x-middle_x+0.5)/middle_x)**2+((y-middle_y+0.5)/middle_y)**2)>1:
+                    board._grid[x, y] = Color.Wall
+                    board._territories[0]._vertices.remove((x, y))
+        return board
 
     def __getitem__(self, name: tuple[int, int]) -> Color:
         if not (isinstance(name, tuple) and len(name) == 2):
@@ -61,80 +79,79 @@ class Board:
     def __repr__(self):
         return f"<{self.__class__.__name__} width={self._grid.shape[0]} height={self._grid.shape[1]}>"
 
-    @staticmethod
-    def other_player(color: Color) -> Color:
-        """Returns the other color (black for white and white for black)"""
-        if color is Color.Empty:
-            raise ValueError("Player cannot be Empty")
-        return Color.Black if color is Color.White else Color.White
+    def next_player(self, player: Optional[Player] = None) -> Player:
+        """Returns the player who is next in the rotation of the game
 
-    def join(self, player: 'Player') -> None:
+        :param player: The reference player. Default to the player currently playing.
+
+        :raises ValueError: The player you gave is not on the board or there is no player on the board
+
+        :returns: The next player"""
+        if not self._players:
+            raise ValueError("No players are joined")
+        if player:
+            if player not in self._players.values():
+                raise ValueError(f"{player} is not joined to this board")
+        else:
+            player = self._current_player
+        players = sorted(self._players.values(), key=lambda p: p.color.value)
+        return players[(players.index(player)+1) % len(players)]
+
+    def join(self, player: Player) -> None:
         """Links a player to the board for a game
 
-        :Parameters:
-            player: :class:`Player`
-                The player to link
+        :param player: The player to link
 
-        :Raises:
-            :class:`ValueError`
-                There are already two players linked"""
-        if not self._players[Color.Black]:
-            self._players[Color.Black] = player
-            player._initiate(self, Color.Black)
-        elif not self._players[Color.White]:
-            self._players[Color.White] = player
-            player._initiate(self, Color.White)
-        else:
-            raise ValueError("Both player are already joined")
+        :raises ValueError: There are already two players linked"""
+        if len(self._players) > max_color:
+            raise ValueError("Board is already full")
+        if player.color in self._players:
+            raise ValueError("The color of this player is already used")
+        if not self._players:
+            self._current_player = player
+        if player.color is None:
+            player._color = next((c for c in Color if c.value > 0 and c not in self._players), None)
+        self._players[player.color] = player
+        player._initiate(board=self)
 
-    def remove_player(self, player: 'Player') -> None:
+    def remove_player(self, player: Player) -> None:
         """Unlinks a player from the board
 
-        :Parameters:
-            player: :class:`Player`
-                The player to unlink
+        :param player: The player to unlink
 
-        :Raises:
-            :class:`ValueError`
-                This player is not linked to the board"""
-        if player is self._players[Color.Black]:
-            self._players[Color.Black]._clear_state()
-            self._players[Color.Black] = None
-        elif player is self._players[Color.White]:
-            self._players[Color.White] = None
-            self._players[Color.White]._clear_state()
-        else:
+        :raises ValueError: This player is not linked to the board"""
+        if player.color not in self._players:
             raise ValueError("Player is not joined")
+        else:
+            del self._players[player.color]
+        if player is self._current_player:
+            self._current_player = list(self._players)[0]
 
     def clear_players(self) -> None:
         """Unlinks all players from the board"""
         for player in self._players.values():
             player._clear_state()
-        self._players = {Color.Black: None, Color.White: None}
+        self._players = {}
 
     @classmethod
-    def from_grid(cls, grid: np.ndarray):
+    def from_grid(cls, grid: np.ndarray) -> Board:
         """Initialize a board from an 2D array of :class:`Color`
 
-        :Parameters:
-            grid: :class:`np.ndarray`
-                A 2D array of :class:`Color` representing the state of the board
+        :param grid: A 2D array of :class:`Color` representing the state of the board
 
-        :Returns:
-            The new created board"""
+        :returns: The new created board"""
         new_board = cls()
         new_board._grid = grid
         new_board._last_grid = np.full(grid.shape, Color.Empty)
         new_board._init_territories()
         return new_board
 
-    def clone(self) -> 'Board':
+    def clone(self) -> Board:
         """Returns a deep copy of the board"""
         new_board = Board()
         new_board.show = self.show
         new_board._grid = np.copy(self._grid)
-        new_board._next_player = self._next_player
-        new_board._prisoners = dict(self._prisoners)
+        new_board._players = dict(self._players)
         new_board._territories = [t.clone(new_board) for t in self._territories]
         return new_board
 
@@ -142,29 +159,26 @@ class Board:
         self._territories = []
         for x in range(self._grid.shape[0]):
             for y in range(self._grid.shape[1]):
-                if not any((x, y) in t._vertices for t in self._territories):
+                if not any((x, y) in t.vertices for t in self._territories):
                     self._territories.append(Territory(x=x, y=y, board=self))
 
     def display(self) -> None:
         """Displays the board as a numpy matrix"""
-        plt.imshow(self.matrix(Color.Black), cmap=cmap, vmin=-1, vmax=1)
+        plt.imshow(self.matrix(), cmap=cmap, vmin=min_color, vmax=max_color)
         plt.pause(0.1)
 
     def is_playable(self, x: int, y: int, color: Color) -> bool:
-        """
-        Checks if a move is valid
+        """Checks if a move is valid
 
-        :Parameters:
-            x: :class:`int`
-                The x coordinate to check
-            y: :class:`int`
-                The y coordinate to check
-            color: :class:`int`
-                The color of the player to check
+        :param x: The x coordinate to check
 
-        :Returns:
-            Indicates if the move is valid
-        """
+        :param y: The y coordinate to check
+
+        :param color: The color of the player to check
+
+        :returns: Indicates if the move is valid"""
+        if not color.is_player():
+            raise ValueError(f"{color.name} is not a player color")
         if self[x, y] is not Color.Empty:
             return False
         grid = np.copy(self._grid)
@@ -192,51 +206,43 @@ class Board:
     def playable_moves(self, color: Color) -> list[tuple[int, int]]:
         """ Gives the list of valid move for a given color
 
-        :Parameters:
-            color: :class:`Color`
-                The player
+        :param color: The player
 
-        :Returns:
-            A list of all vertices where the player can play
+        :returns: A list of all vertices where the player can play
         """
         playable = []
         for t in self._territories:
             if t.color is Color.Empty:
-                for x, y in t._vertices:
+                for x, y in t.vertices:
                     if self.is_playable(x, y, color):
                         playable.append((x, y))
         return playable
 
-    def run_game(self, max_turn: Optional[int] = 1000, max_duration: Optional[int] = None) -> 'Player':
+    def run_game(self, max_turn: Optional[int] = 1000, max_duration: Optional[int] = None) -> Player:
         """Runs a game on this board between two players. The players have to be linked to the board with :func:`join` before
 
-        :Parameters:
-            max_turn: Optional[:class:`int`]
-                The maximum number of move before ending the game
-            max_duration: Optional[:class:`int`]
-                The maximum number of seconds before ending the game
+        :param max_turn: The maximum number of move before ending the game
 
-        :Raises:
-            :class:`ValueError`
-                Not enough players to start the game
-            :class:`TypeError`
-                A player returns an invalid move type
+        :param max_duration: The maximum number of seconds before ending the game
 
-        :Returns:
-            The player who wins the game"""
-        if any(player is None for player in self._players):
-            raise ValueError("The board needs two players to be run")
+        :raises ValueError: Not enough players to start the game
+
+        :raises TypeError: A player returns an invalid move type
+
+        :returns: The player who wins the game"""
+        if len(self._players) < 2:
+            raise ValueError("The board needs at least two players to be run")
         if max_turn is None and max_duration is None:
             warnings.warn("max_turn and max_duration are both to None, game might run forever")
         c = 0
         starting_time = time.time()
         while (not c or c < max_turn) and (not max_duration or time.time()-starting_time < max_duration):
-            move = self._players[self._next_player].play()
+            move = self._current_player.play()
             if move is None:
-                if self.skip(player=self._next_player):
+                if self.skip(color=self._current_player.color):
                     return self.winner()
             elif isinstance(move, (tuple, list, np.ndarray)) and len(move) == 2:
-                self.play(*move, color=self._next_player)
+                self.play(*move, color=self._current_player.color)
             else:
                 raise TypeError("play method must return None or a 2-tuple")
             c += 1
@@ -245,26 +251,22 @@ class Board:
     def play(self, x: int, y: int, *, color: Color) -> None:
         """Play a move manually without using Player object
 
-        :Parameters:
-            x: :class:`int`
-                The x coordinate of the move to play
-            y: :class:`int`
-                The y coordinate of the move to play
-            color: :class:`Color`
-                The color of the move to play
+        :param x: The x coordinate of the move to play
 
-        :Raises:
-            :class:`ValueError`
-                The move is invalid, or it's the wrong player"""
-        if color is not self._next_player:
-            raise ValueError('Wrong player')
+        :param y: The y coordinate of the move to play
+
+        :param color: The color of the move to play
+
+        :raises ValueError: The move is invalid, or it's the wrong player"""
+        self._verify_color_before_playing(color)
 
         if not self.is_playable(x, y, color):
             raise ValueError('You cannot play here')
 
         self._last_grid = np.copy(self._grid)
         self._grid[x, y] = color
-        self._next_player = self.other_player(self._next_player)
+        if self._players:
+            self._current_player = self.next_player()
 
         modified = [t for t in self.territories(color) if t.is_touching(x, y)]
         for t in self._territories:
@@ -275,11 +277,13 @@ class Board:
                 self._territories.remove(t)
             self._territories.append(merge_territory)
 
-        for t in self.territories(self.other_player(color)):
-            if not t.freedom:
-                self._territories.remove(t)
-                for i, j in t._vertices:
-                    self._grid[i, j] = 0
+        for t in self.territories():
+            if t.color.is_player() and (t.color is not color) and not t.freedom():
+                t._color = Color.Empty
+                if color not in self._prisoners:
+                    self._prisoners[color] = 0
+                for i, j in t.vertices:
+                    self._grid[i, j] = Color.Empty
                     self._prisoners[color] += 1
 
         if not any(t.includes(x, y, color) for t in self._territories):
@@ -288,29 +292,40 @@ class Board:
         if self.show:
             self.display()
 
-    def skip(self, *, player: Color, show: Optional[bool] = None) -> Optional[bool]:
+    def skip(self, *, color: Color) -> bool:
         """Skip a turn manually without using Player object
 
-            :Raises:
-                :class:`ValueError`
-                    It's the wrong player"""
-        if player is not self._next_player:
-            raise ValueError('Wrong player')
-        if np.all(self._last_grid is self._grid):
-            return True
-        self._next_player = self.other_player(player)
-        if show is True or (show is None and self.show):
-            self.display()
+        :param color: The color of the move to play
 
-    def winner(self) -> 'Player':
+        :raises ValueError: It's the wrong player
+
+        :returns: True if the game is over because it's the second skip in a row, False otherwise"""
+
+        self._verify_color_before_playing(color)
+        if np.all(self._last_grid == self._grid) and not np.all(self._grid == Color.Empty):
+            return True
+        if self._players:
+            self._current_player = self.next_player()
+        self._last_grid = np.copy(self._grid)
+        if self.show:
+            self.display()
+        return False
+
+    def _verify_color_before_playing(self, color):
+        if not color.is_player():
+            raise ValueError(f"{color.name} is not a player color")
+        if self._players:
+            if color not in self._players:
+                warnings.warn(f'{color.name} is not the color of a joined player')
+            if color is not self._current_player.color:
+                warnings.warn(f'The {color.name} player  is not supposed to play now')
+
+    def winner(self) -> Player:
         """Returns the current winner of the board by comparing the scores of both player
         In case of equality, White wins
 
-        :Returns:
-            The player who currently leads the game"""
-        if self.score(Color.Black) > self.score(Color.White):
-            return self._players[Color.Black]
-        return self._players[Color.White]
+        :returns: The player who currently leads the game"""
+        return max(reversed(self._players.values()), key=lambda p: self.score(p.color))
 
     def around(self,
                x: int,
@@ -319,16 +334,13 @@ class Board:
                ) -> Generator[tuple[int, int], None, None]:
         """A quick way to get vertices around a given point
 
-        :Parameters:
-            x: :class:`int`
-                The x coordinate of the point
-            y: :class:`int`
-                The y coordinate of the point
-            include_center: :class:`bool`
-                Wether to include the given point or not
+        :param x: The x coordinate of the point
 
-        :Yields:
-            The points around"""
+        :param y: The y coordinate of the point
+
+        :param include_center: Wether to include the given point or not
+
+        :yields: The points around"""
         if x > 0:
             yield x - 1, y
         if y > 0:
@@ -343,40 +355,23 @@ class Board:
     def prisoners(self, color: Color) -> int:
         """Get the number of prisoners owned by a player
 
-        :Parameters:
-            color: :class:`Color`
-                The color of the player
+        :param color: The color of the player
 
-        :Returns:
-            The number of prisoners"""
-        return self._prisoners[color]
+        :returns: The number of prisoners"""
+        return self._prisoners.get(color, 0)
 
-    def matrix(self, color: Color) -> np.ndarray:
-        """Returns the current state of the board as a numpy matrix
+    def matrix(self) -> np.ndarray:
+        """Returns the current state of the board as a numpy matrix to facilitate move calculation
 
-        * 0 is an empty vertice
-        * 1 is a vertice the player owns
-        * -1 is a vertice owned by the opponent
-
-        :Parameters:
-            color: :class:`Color`
-                The color of the player
-
-        :Returns:
-            The matrix representing the board
-        """
-        grid = np.vectorize(lambda x: x.value)(self._grid)
-        return -grid if color is Color.White else grid
+        :returns: The matrix representing the board"""
+        return np.vectorize(lambda c: c.value)(self._grid)
 
     def territories(self, color: Optional[Color] = None) -> list[Territory]:
         """Returns territories currently on the board. If a color is specified, only territories of the given color are returned
 
-        :Parameters:
-            color: Optional[:class:`Color`]
-                The color of the territories to get
+        :param color: The color of the territories to get
 
-        :Returns:
-            A list of territories"""
+        :returns: A list of territories"""
         if color is None:
             return self._territories
         else:
@@ -388,14 +383,11 @@ class Board:
                       ) -> Optional[Territory]:
         """Get a territory from a vertice
 
-        :Parameters:
-            x: :class:`int`
-                The x coordinate of the territory to get
-            y: :class:`int`
-                The y coordinate of the territory to get
+        :param x: The x coordinate of the territory to get
 
-        :Returns:
-            The territory which owns the vertice if any"""
+        :param y: The y coordinate of the territory to get
+
+        :returns: The territory which owns the vertice if any"""
         for t in self._territories:
             if t.includes(x, y):
                 return t
@@ -404,12 +396,9 @@ class Board:
     def vertices(self, color: Color) -> list[tuple[int, int]]:
         """Get all vertices from a given color
 
-        :Parameters:
-            color: :class:`Color`
-                The color of the vertices to get
+        :param color: The color of the vertices to get
 
-        :Returns:
-            The list of vertices"""
+        :returns: The list of vertices"""
         vertices = []
         for x in range(self._grid.shape[0]):
             for y in range(self._grid.shape[1]):
@@ -420,10 +409,7 @@ class Board:
     def score(self, color: Color) -> int:
         """Returns the score of a player i.e. the number of vertices belonging to the player + the number of his prisoners
 
-        :Parameters:
-            color: :class:`Color`
-                The color of the player
+        :parameters player: The color of the player
 
-        :Returns:
-            The score of the given player"""
-        return self._prisoners[color] + np.count_nonzero(self._grid is color)
+        :returns: The score of the given player"""
+        return self._prisoners.get(color, 0) + np.count_nonzero(self._grid == color)
